
//  -----------------------------------------------------------------
//  rrg.js  -   a collection of JavaScript routines that are used in
//  Resource File Reports
//
//  Copyright (C) 2003 Echelon Corporation
//  All Rights Reserved
//  -----------------------------------------------------------------

// The following variables hold JavaScript functions. An OnInit() function
// (further down in this file) that is called whenever the HTML page finished
// loading determines the best possible set of implementations and initializes
// these function "pointers" accordingly. 
// See MsTransform, below, for a more detailed discussion and reference 
// implementation.
// When using features that are defined within this file, always refer to those
// via these function pointers:
var Transform;      // (void) Transform( string xml-uri, string xsl-uri [, string xpath-expression] )
var GetNodeValue;   // String GetNodeValue( w3cdom:Node node )

function Init() {
    // detect the browser and set the function pointers accordingly:

    var browser = navigator.userAgent.toLowerCase();
    var version = 0;

    function GetVersion( id, iPos ) {
        while ( (id.charAt(iPos) == ' ') || (id.charAt(iPos) == '/') ) {
            ++iPos;
        }
        return id.charCodeAt(iPos) - '0'.charCodeAt(0);
    }

    Transform = NotImplemented;
    GetNodeValue = _W3C_GetNodeValue;

    var iPosition = browser.indexOf('konqueror');
    if ( iPosition >= 0 ) {
        // Konqueror is the standard browser for many Linux/KDE users. 
        // Change the hook to add Konqueror support. When adding support for a new 
        // browser, also remember to update the list of supported browsers in the 
        // NotImplemented() function.

        // enable thje following line if version-dependent konqueror support is required:
        // version = GetVersion(browser, iPosition+'konqueror'.length);
        // Transform = .....
    } else {
        iPosition = browser.indexOf('opera');
        if ( iPosition >= 0 ) {
            // Opera must be tested prior to the "big ones", as Opera allows for
            // user-defined browser-identifiation strings. Thus, Opera can masquerade
            // as a Mozilla-compatible browser without having that compatibility at all. 
            // However, Opera will always (?) leave the work "opera" in the identification
            // string, allowing us to filter it out. 
            // Change the hook to add Opera support. When adding support for a new 
            // browser, also remember to update the list of supported browsers in the 
            // NotImplemented() function.

            // enable thje following line if version-dependent opera support is required:
            // version = GetVersion(browser, iPosition+'opera'.length);
            // Transform = ....
         } else {
            iPosition = browser.indexOf('msie');
            if ( iPosition >= 0 ) {
                // Microsoft Internet Explorer. Get the version number:
                version = GetVersion(browser, iPosition+'msie'.length);
               if ( version >= 6 ) {
                    Transform = _MsTransform;
                }
                GetNodeValue = _MS_GetNodeValue;
            } else {
                iPosition = browser.indexOf('mozilla');
                if ( (iPosition >= 0) && (browser.indexOf('compatible') < 0) ) {
                    // assume Netscape or genuine Mozilla. Note most browsers will advertise themselves as
                    // "Mozilla [...] (compatible: <actual browser>)". Thus, we cannot rely on the presence
                    // of the word "mozilla" alone.

                    version = GetVersion(browser, 'mozilla'.length);
                    if ( document.implementation.hasFeature("XPath", "3.0") ) {

                        // Note Mozilla-specific transformation routines will be enabled here. A draft version
                        // of source a routine is provided below, see _MozillaTransform, for details.
                        // Due to Mozilla not yet supporting the required features in full, this transformation
                        // has not been enabled and "NotImplemented" is used instead.

                        Transform = NotImplemented; // _MozillaTransform;
                    }
                }
            }
        }
    }
}

// NotImplemented() is the "transformation handler" for unsupported browsers, providing
// a verbose description of the problem, plus an optional alternative approach via the
// _GenericTransform() routine. See there for more details.
function NotImplemented(xml, xsl, filter) {
    var genericHandler = 
        confirm("The required function to transform a single node has not has not yet been\n" + 
                "implemented for your browser. Alternatively to using a fully supported\n" + 
                "browser as listed below, you may choose to browse the report using a\n" +
                "generic and unselective transformation approach. Note only parts of the\n" +
                "report will be accessible using this alternative approach.\n" +
                "Click Ok to use the alternative transformation approach, or Cancel\n" +
                "to return to the current page.\n\n" +
                "Currently fully supported web browser(s) are:\n" +
                "Microsoft Internet Explorer 5.5 (or better)");
    if ( genericHandler ) {
        Transform = _GenericTransform;
        Transform( xml, xsl, filter );
    }
}

function _ReportError(e,xml,xsl,filter) {
    alert(  "An error occurred when trying to locate a resource\n" +
             e + "\n" +
            "XML datasource: " + xml + "\n" +
            "Transformation: " + xsl + "\n" +
            "XPath phrase:   " + filter + "\n" );
}

// use report to extract a node out of the source document,
// using the XPath filter expression 'filter', 
// and to apply the XSL transformation. The result is written
// to the document.
// The filter can be an empty string, in which case the 
// entire XML is transformed.
// A filter example (ignoring the namespace for brevity): 
// "/report/items/item[@id='SNVT_amp']" - note the nested quotes!
// In the typical use case, the Transform function
// is being called through the href attribute of a HTML <a> 
// tag, e.g. <a href="javascript:Transform(....)">Click here</a>
// The caller must be careful to choose the correct pairing of
// single and/or double quotes, combined with the entities &apos;
// and &quot; to achieve the correct results when calling this 
// function.
//
function _MsTransform ( xml, xsl, filter ) {

    function _MsGetDocument(uri) {
        var oDocument = new ActiveXObject("Microsoft.XMLDOM"); 
        oDocument.async = false;
        oDocument.load(uri);
        return oDocument;
    }

    try {
        var oXml = _MsGetDocument(xml);
        var oXsl = _MsGetDocument(xsl);

        var oNode;
        if ( filter == "" ) {
            oNode = oXml;
        } else {
            oXml.setProperty("SelectionLanguage", "XPath");
            oNode = oXml.selectSingleNode(filter);
        }

        if ( !oNode ) {
            alert("No such resource: " + src + "#xpointer(" + filter + ")" );
        } else {
            document.open();
            var sResult =  oNode.transformNode( oXsl );
            document.write( sResult );
            document.close();
        }
    } catch ( e ) {
        _ReportError(e.description,xml,xsl,filter);
    }
}

// _MozillaTransform() is a Transform()-like routine that uses the XSLTProcessor
// Javascript binding; see http://mozilla.org/projects/xslt/js-interface.html 
// for details of that interface and technology.
function _MozillaTransform( xml, xsl, xpath ) {

    try {

        var processor = new XSLTProcessor();

        var data = document.implementation.createDocument("", xml, null);
        var stylesheet = document.implementation.createDocument("", xsl, null);

        // onXmlLoaded() is the last "load" event handler. When this function executes, both
        // the XML data and the XSL transformation stylesheet has been loaded. The onXmlLoaded()
        // handler is thus the code of the Mozilla / DOM 3 implementation, as it performs the
        // actual transformation and display of the result:
        function onXmlLoaded() {
            processor.importStylesheet(stylesheet);

            var result;
            if ( xpath.length == 0 ) {
                // no xpath given: apply transformation to entire XML document:
                result = processor.transformToDocument(data);
            } else {

                // NOTE:
                // The following code shows how this transformation will evenutally work with
                // browsers that support DOM Level 3 for JavaScript; see the W3C's site for
                // details (http://www.w3.org/TR/2002/WD-DOM-Level-3-XPath-20020328/xpath.html).
                // However, at the time of writing, this doesn't work with the Mozilla Browser
                // (www.mozilla.org), which states to implement support for the required
                // features (document.implementation.hasFeature("XPath", "3.0") returns true).
                // At the time of writing, however, the mozilla community states the level
                // of DOM 3 support is "in progress", with no firm date set for completion.
                // Any hints or notifications are most welcome; please send email to 
                // lonsupport@echelon.com. Please make sure to refer to the Resource Report
                // Pages (Resource Report Generator) for further internal reference. 

                // some xpath given: filter XML data with this XPath, then apply
                // transformation on the XPath result:
                var oNamespaceResolver = data.createNSResolver(data);
                var oResult = data.evaluate(xpath, data, oNamespaceResolver, XPathResult.ANY_TYPE, null); 

                result = processor.transformToDocument(oResult.singleNodeValue);
            }

            // replace (HTML) DOM in current browser window with new content:
            document.body = result.body;
            document.title = xml + "#" + xpath;
        }

        // onXslLoaded() completes the loading of the XSL stylesheet, and starts
        // loading the XML data:
        function onXslLoaded() {
            data.addEventListener("load", onXmlLoaded, false);
            data.load(xml);
        }

        stylesheet.addEventListener("load", onXslLoaded, false);
        stylesheet.load(xsl);

    } catch ( e ) {
        _ReportError(e.description,xml,xsl,xpath);
    }
}

// _GenericTransform() can be enabled upon user request. It ignores the 
// XSL stylesheet provided as well as the XPath filter. It simply replaces
// the current window with the XML provided, counting on the browser's 
// correct implementation and interpretation of the xml-stylesheet tag
function _GenericTransform( xml, xsl, filter ) {
    try {
        location.replace(xml);
    } catch ( e ) {
        _ReportError(e.description,xml,xsl,filter);
    }
}

// Back() provides the functionality of the browser's "back" button. 
function Back() {
    history.back();
}

function _W3C_GetNodeValue ( node ) {
    return node.nodeValue;
}

function _MS_GetNodeValue ( node ) {
    return node.innerText;
}

function FindResource() {

    function Strip( s ) {
        var sStart = s.charAt(0);
        while ( sStart == ' ' || sStart == '"' || sStart == "'" ) {
            s = Strip( s.substr(1,s.length-1) );
            sStart = s.charAt(0);
        }
        var sEnd = s.charAt(s.length-1);
        while ( (sEnd == ' ') || (sEnd == '"') || (sEnd == "'") ) {
            s = Strip( s.substr(0, s.length-1) );
            sEnd = s.charAt(s.length-1);
        }
        return s;
    }

    var sToFind = document.ResourceLocator.ResourceName.value;
    sToFind = sToFind.toLowerCase();

    if ( sToFind.length == 0 ) {
        var iConfirmed = confirm( "You have not provided a keyword to search for.\n" +
                                  "The search will thus list all resources of this type\n" +
                                  "on a single page.\n" +
                                  "Note that creating this page might take a while.\n\n" +
                                  "Do you want to continue?" );
        if ( !iConfirmed ) {
            return;
        }
    }

    var aLinks = document.getElementsByTagName("a");
    var bFound = false;
    var sItemList = "";

    for ( var i = 0; i < aLinks.length; ++i ) {
        var oNode = aLinks.item(i);
        var sLabel = GetNodeValue(aLinks.item(i));
        sLabel = sLabel.toLowerCase();
        if ( sLabel.match(sToFind) ) {
            var sLink = unescape(oNode.getAttribute("href"));
            var iOpen = sLink.indexOf('(');
            var iClose = sLink.indexOf(')');

            if ( iOpen > 0 && iOpen < iClose ) {
                var sArguments = sLink.substr(iOpen+1, iClose-iOpen-1);
                var aDetails = sArguments.split(',');
                if ( aDetails.length == 3 ) {
                    var xml = Strip(aDetails[0]);
                    var xsl = Strip(aDetails[1]);
                    var xpath = Strip(aDetails[2]);
                    bFound = true;

                    Transform( xml, xsl, xpath );
                    break;
                }
            }

        }
    }
    if (!bFound) {
        alert( "No such resource: " + sToFind );
    }
}
